/*
 * nTupleReader.cpp
 *
 *  Created on: Feb 13, 2012
 *      Author: daniel
 */




#include "nTupleReader.h"
#include <string>
#include <vector>
#include <iostream>
#include "weightFunctions.h"
#include "pdf.h"
#include "LHAPDF/LHAPDF.h"


nTupleReader::nTupleReader(): RootFileReaderBase(){
	init(d_NI,"BHSntuples");
	d_member=-1;
	d_fin->SetBranchAddress("alphasPower",&d_alphasPower);
	d_fin->SetBranchAddress("part",d_part);
};


void nTupleReader::setPDF(const std::string& name){
	static bool hasDoneInit=false;
	if (!hasDoneInit){
		LHAPDF::initLHAPDF();
		hasDoneInit=true;
	}
	LHAPDF::initPDFSetByName(name);
	LHAPDF::initPDF(0);
}

void nTupleReader::setPDFmember(int member){
	if (member!=d_member){
		LHAPDF::initPDF(member);
		d_member=member;
	}
}

double nTupleReader::computeWeight(double newFacScale,double newRenScale){
	switch ( getType() ){
	case 'B': return computeBornWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'I':  return computeVsubWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'V': case 'L': return computeLoopWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'R': return computeBornWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	}

};

double nTupleReader::computeWeight2(double newFacScale,double newRenScale){
	switch ( getType() ){
	case 'B': return computeBornWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'I':  return computeVsubWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'V': case 'L': return computeLoopWeight(d_NI,newFacScale,newRenScale,getAlphasPower());
	case 'R': return computeRealWeightWithProperStat(d_NI,newFacScale,newRenScale,getAlphasPower());
	}
};


void nTupleReader::setPP(){
	Pdf::initialState::setInitialState(1,1);
}
void nTupleReader::setPPbar(){
	Pdf::initialState::setInitialState(1,-1);
}



