/*
 * weightFunctions.cpp
 *
 *  Created on: Jan 13, 2012
 *      Author: daniel
 */

#include "weightFunctions.h"
#include "pdf.h"
#include <cmath>
#include "LHAPDF/LHAPDF.h"



double computePDFFactor(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newScale){
	double oldPdfWgt = Pdf::pdf(Ev.x1,Ev.muF,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,Ev.muF,Ev.id2,Pdf::initialState::s_id2) ;
	double newPdfWgt = Pdf::pdf(Ev.x1,newScale,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,newScale,Ev.id2,Pdf::initialState::s_id2) ;
	double pdfFactor=newPdfWgt/oldPdfWgt;
	return pdfFactor;
}

double computeAlphaFactor(NtupleInfo<MAX_NBR_PARTICLES>& NI,double newScale,int alphasPower) {
	double oldAlpha=NI.alphas;
	double newAlpha=LHAPDF::alphasPDF(newScale);
	double alphasFactor = std::pow(newAlpha/oldAlpha,alphasPower);
	return alphasFactor;
}

double computeBornWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double alphaFactor){
	double newPdfWgt = Pdf::pdf(Ev.x1,newFacScale,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,newFacScale,Ev.id2,Pdf::initialState::s_id2) ;
	return Ev.me_wgt*alphaFactor*newPdfWgt;
};


double computeBornWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,int alphasPower){
	double alphaFactor=computeAlphaFactor(Ev,newRenScale,alphasPower);
	double newPdfWgt = Pdf::pdf(Ev.x1,newFacScale,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,newFacScale,Ev.id2,Pdf::initialState::s_id2) ;
	return Ev.me_wgt*alphaFactor*newPdfWgt;
}


double computeLoopWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,int alphasPower){
	double alphaFactor=computeAlphaFactor(Ev,newRenScale,alphasPower);
	return computeLoopWeight(Ev,newFacScale,newRenScale,alphaFactor);
}

double computeLoopWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,double alphaFactor){
	Pdf::pdfArray X1,X2,X1overX1p,X2overX2p;
	Pdf::LHAComputePdf(Ev.x1,newFacScale,X1);
	Pdf::LHAComputePdf(Ev.x2,newFacScale,X2);

	double L=2.0*log(newRenScale/Ev.muR);
	double res2=
		pdf(X1,Ev.id1,Pdf::initialState::s_id1)*pdf(X2,Ev.id2,Pdf::initialState::s_id2) *
		(
			Ev.me_wgt
			+Ev.usr_wgts[0]*L
			+0.5*Ev.usr_wgts[1]*L*L
		);
	return res2*alphaFactor;
}


double computeVsubWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,int alphasPower){
	double alphaFactor=computeAlphaFactor(Ev,newRenScale,alphasPower);
	return computeVsubWeight(Ev,newFacScale,newRenScale,alphaFactor);
}

double computeVsubWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,double alphaFactor){
	Pdf::pdfArray X1,X2,X1overX1p,X2overX2p;
	Pdf::LHAComputePdf(Ev.x1,newFacScale,X1);
	Pdf::LHAComputePdf(Ev.x2,newFacScale,X2);
	Pdf::LHAComputePdf(Ev.x1/Ev.x1p,newFacScale,X1overX1p);
	Pdf::LHAComputePdf(Ev.x2/Ev.x2p,newFacScale,X2overX2p);

	double L=2.0*log(newRenScale/Ev.muR);
	double LF=2.0*log(newFacScale/Ev.muF);

	double res2=
		pdf(X1,Ev.id1,Pdf::initialState::s_id1)*pdf(X2,Ev.id2,Pdf::initialState::s_id2) *
		(
			Ev.me_wgt
			+Ev.usr_wgts[0]*L
			+0.5*Ev.usr_wgts[1]*L*L
		)
	 +
		(
		pdf_vsub1(X1,Ev.id1,Pdf::initialState::s_id1)*(Ev.usr_wgts[2]+Ev.usr_wgts[10]*LF)
		+pdf_vsub2(X1overX1p,Ev.x1p,Ev.id1,Pdf::initialState::s_id1)*(Ev.usr_wgts[3]+Ev.usr_wgts[11]*LF)
		+pdf_vsub3(X1,Ev.id1,Pdf::initialState::s_id1)*(Ev.usr_wgts[4]+Ev.usr_wgts[12]*LF)
		+pdf_vsub4(X1overX1p,Ev.x1p,Ev.id1,Pdf::initialState::s_id1)*(Ev.usr_wgts[5]+Ev.usr_wgts[13]*LF)
		)* pdf(X2,Ev.id2,Pdf::initialState::s_id2) +
		(
		pdf_vsub1(X2,Ev.id2,Pdf::initialState::s_id2)*(Ev.usr_wgts[6]+Ev.usr_wgts[14]*LF)
		+pdf_vsub2(X2overX2p,Ev.x2p,Ev.id2,Pdf::initialState::s_id2)*(Ev.usr_wgts[7]+Ev.usr_wgts[15]*LF)
		+pdf_vsub3(X2,Ev.id2,Pdf::initialState::s_id2)*(Ev.usr_wgts[8]+Ev.usr_wgts[16]*LF)
		+pdf_vsub4(X2overX2p,Ev.x2p,Ev.id2,Pdf::initialState::s_id2)*(Ev.usr_wgts[9]+Ev.usr_wgts[17]*LF)
		)* pdf(X1,Ev.id1,Pdf::initialState::s_id1) ;




	return res2*alphaFactor;
}

double computeRealWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double alphaFactor){
	double newPdfWgt = Pdf::pdf(Ev.x1,newFacScale,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,newFacScale,Ev.id2,Pdf::initialState::s_id2) ;
	return Ev.me_wgt*alphaFactor*newPdfWgt;
}

double computeRealWeight(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,int alphasPower){
	double alphaFactor=computeAlphaFactor(Ev,newRenScale,alphasPower);
	return computeRealWeight(Ev,newFacScale,alphaFactor);
}

double computeRealWeightWithProperStat(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double alphaFactor){
	double newPdfWgt = Pdf::pdf(Ev.x1,newFacScale,Ev.id1,Pdf::initialState::s_id1)*Pdf::pdf(Ev.x2,newFacScale,Ev.id2,Pdf::initialState::s_id2) ;
	return Ev.me_wgt2*alphaFactor*newPdfWgt;
}

double computeRealWeightWithProperStat(NtupleInfo<MAX_NBR_PARTICLES>& Ev,double newFacScale,double newRenScale,int alphasPower){
	double alphaFactor=computeAlphaFactor(Ev,newRenScale,alphasPower);
	return computeRealWeightWithProperStat(Ev,newFacScale,alphaFactor);
}

